package com.sxtvs.open.api.youmei.service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import com.sxtvs.open.api.youmei.dto.CheckWordResponse;
import com.sxtvs.open.api.youmei.entity.YoumeiAccount;
import com.sxtvs.open.api.youmei.mappser.YoumeiAccountMapper;
import com.sxtvs.open.core.sls.AliyunLogger;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.hc.client5.http.fluent.Request;
import org.apache.hc.core5.http.message.BasicNameValuePair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Service
@Slf4j
public class YoumeiAccountServiceImpl extends ServiceImpl<YoumeiAccountMapper, YoumeiAccount> implements IYoumeiAccountService {

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private AliyunLogger logger;


    private final HashMap<Integer, String> belongMap = new HashMap<>() {{
        put(9, "错别字、词");
        put(31, "多字错误");
        put(32, "少字错误");
        put(35, "语义重复");
        put(34, "语序错误");
        put(39, "量和单位差错");
        put(36, "数字差错");
        put(20, "句式杂糅");
        put(21, "标点符号差错");
        put(24, "句子查重");
        put(23, "英文校对");
        put(119, "重要讲话引用");
        put(101, "姓名和职务信息");
        put(123, "地理名词");
        put(19, "机构名称");
        put(124, "专有名词及术语");
        put(8, "时政重点词");
        put(122, "媒体报道禁用词和慎用词");
        put(240, "法律法规名称");
        put(6, "常识差错");
        put(105, "涉国家统一、主权和领土完整");
        put(109, "涉民族宗教");
        put(112, "涉黄、暴、恐、赌、毒");
        put(111, "涉低俗辱骂");
        put(108, "涉违法违规");
        put(118, "其他敏感内容");
        put(2001, "自定义禁用词");
        put(2002, "自定义敏感词");
        put(2003, "自定义正词");
        put(2004, "自定义错词");
        put(2005, "自定义重点词");
        put(2006, "自定义领导人职务信息");
        put(2007, "自定义领导人排序");
    }};

    private final Cache<String, Object> cache = Caffeine.newBuilder()
            .expireAfterWrite(10, TimeUnit.MINUTES)
            .maximumSize(10000)
            .build();


    private final Cache<String, String> tokenCache = Caffeine.newBuilder()
            .expireAfterWrite(1, TimeUnit.MINUTES)
            .build();


    public String getWbjcTokenCache() {
        return tokenCache.get("sxgdwbjc", this::getWbjcToken);
    }

    public String getWbjcToken(String accountName) {
        return this.lambdaQuery()
                .eq(YoumeiAccount::getOpenId, accountName)
                .one().getAccessToken();
    }

    @SneakyThrows
    public Object wordCheck(String text) {

        var body = Request.post("https://api-open-wx-www.yqt365.com/dataapp/api/umei/fw/open/wbjc/article_correct_external")
                .bodyForm(List.of(new BasicNameValuePair("accessToken", getWbjcTokenCache()),
                        new BasicNameValuePair("text", text)), StandardCharsets.UTF_8)
                .execute()
                .returnContent()
                .asString();

        logger.info("key", "checkWordResponse", "data", body);

        CheckWordResponse checkWordDto = objectMapper.readValue(body, CheckWordResponse.class);
        CheckWordResponse.DataDTO data = checkWordDto.getData();

        if (data.getChecklist().size() == 0) {
            data.setRawText(text);
            data.setReplaceText(text);
            return checkWordDto.getData();
        }

        replaceText(text, data);

        return checkWordDto.getData();
    }

    private void replaceText(String text, CheckWordResponse.DataDTO data) {
        StringBuilder replaceBuilder = new StringBuilder();
        StringBuilder rawBuilder = new StringBuilder();
        int index = 0;
        for (CheckWordResponse.DataDTO.ChecklistDTO checklistDTO : data.getChecklist()) {
            Integer position = checklistDTO.getPosition();
            int length = checklistDTO.getWordHtml().length();

            CheckWordResponse.DataDTO.ChecklistDTO.TypeDTO typeDTO = checklistDTO.getType();
            Integer belongId = typeDTO.getBelongId();
            checklistDTO.setYoumeiWordName(belongMap.getOrDefault(belongId, typeDTO.getName()));


            replaceBuilder.append(text, index, position);
            rawBuilder.append(text, index, position);
            index += (position - index + length);

            String suggest = checklistDTO.getSuggest().stream().findFirst().orElse("");

            replaceBuilder.append(suggest);
            String wrapRed = wrapRed(text.substring(position, position + length), rawBuilder.length());
            checklistDTO.setColorPosition(rawBuilder.length());
            rawBuilder.append(wrapRed);
            checklistDTO.setColorLength(wrapRed.length());

        }
        rawBuilder.append(text, index, text.length());
        replaceBuilder.append(text, index, text.length());
        data.setRawText(rawBuilder.toString());
        data.setReplaceText(replaceBuilder.toString());
    }

    private String wrapRed(String text, int position) {
        return "<span  class=\"selectSpan\" data-umpos=\"" + position + "\">" + text + "</span>";
    }

    public Object wordCheckCache(String text) {
        return cache.get(text, this::wordCheck);
    }

    public static void main(String[] args) throws IOException {

    }

}
